/*******************************************************************************
 *
 * blueMarine - open source photo workflow
 * =======================================
 *
 * Copyright (C) 2003-2010 by Fabrizio Giudici
 * Project home page: http://bluemarine.tidalwave.it
 *
 *******************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *******************************************************************************
 *
 * $Id$
 *
 ******************************************************************************/
package it.tidalwave.openrdf.elmo;

import javax.annotation.Nonnull;
import javax.annotation.Nullable;
import org.openrdf.elmo.ElmoManager;

/*******************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id$
 *
 ******************************************************************************/
public class ElmoManagerThreadLocal
  {
    private ElmoManagerThreadLocal()
      {
      }
    
    private final static ThreadLocal<ElmoManager> tl = new ThreadLocal<ElmoManager>();

    @Nonnull
    public static ElmoManager get()
      throws IllegalStateException
      {
        final ElmoManager em = tl.get();

        if (em == null)
          {
            throw new IllegalStateException("No ElmoManager bound to the current thread: " + Thread.currentThread());
          }

        return em;
      }

    public static void set (@Nullable final ElmoManager elmoManager)
      {
        tl.set(elmoManager);
      }
  }
