
/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package it.tidalwave.openrdf.elmo;

import javax.annotation.CheckForNull;
import javax.annotation.Nonnegative;
import javax.annotation.Nonnull;
import java.beans.PropertyChangeListener;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Locale;
import java.util.HashMap;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Set;
import java.util.SortedSet;
import java.util.TreeSet;
import org.openrdf.elmo.Entity;
import org.openrdf.concepts.skos.core.Concept;
import it.tidalwave.util.As;
import it.tidalwave.util.Finder;
import it.tidalwave.util.Finder.SortCriterion;
import it.tidalwave.util.Finder.SortDirection;
import it.tidalwave.util.NotFoundException;
import it.tidalwave.semantic.SkosConcept;
import it.tidalwave.semantic.SkosConceptUtils;
import it.tidalwave.semantic.Wrapper;
import static it.tidalwave.openrdf.elmo.impl.Utils.*;
        
/***********************************************************************************************************************
 *
 * @author Fabrizio Giudici
 * @version $Id$
 *
 **********************************************************************************************************************/
public class ElmoSkosConcept implements SkosConcept
  {
    @Nonnull
    private final As object;

    private static abstract class FinderHelper<T> implements Finder<T>
      {
        @Nonnull
        private final List<T> results = new ArrayList<T>();

        @CheckForNull
        private Class<T> type;

        private boolean computed = false;

        public FinderHelper()
          {
          }

        @Override
        public int count()
          {
            compute();
            return results.size();
          }

        @Override
        @Nonnull
        public Finder<T> from (final @Nonnegative int firstResult)
          {
            throw new UnsupportedOperationException("Not supported yet.");
          }

        @Override
        @Nonnull
        public Finder<T> max (final @Nonnegative int maxResults)
          {
            throw new UnsupportedOperationException("Not supported yet.");
          }

        @Override
        @Nonnull
        public T result()
          throws NotFoundException
          {
            compute();

            switch (results.size())
              {
                case 0: throw new NotFoundException();
                case 1: return results.get(0);
                default: throw new RuntimeException("Unexpected multiple results!");
              }
          }

        @Override
        @Nonnull
        public T firstResult()
          throws NotFoundException
          {
            compute();

            if (results.isEmpty())
              {
                throw new NotFoundException();
              }

            return results.get(0);
         }

        @Override
        @Nonnull
        public List<? extends T> results()
          {
            compute();
            return results;
          }

        @Override
        @Nonnull
        public Finder<T> sort (final @Nonnull SortCriterion criterion, final @Nonnull SortDirection direction)
          {
            throw new UnsupportedOperationException("Not supported yet.");
          }

        @Override
        @Nonnull
        public Finder<T> sort (final @Nonnull SortCriterion criterion)
          {
            throw new UnsupportedOperationException("Not supported yet.");
          }

        @Override
        @Nonnull
        public <X> Finder<X> ofType (final @Nonnull Class<X> type)
          {
            this.type = (Class<T>)type;
            return (Finder<X>)this;
          }

        private synchronized void compute()
          {
            if (!computed)
              {
                results.addAll(filtered(doCompute(), type));
                computed = true;
              }
          }

        @Nonnull
        protected abstract Set<Concept> doCompute();
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    public ElmoSkosConcept (final @Nonnull As object)
      {
        this.object = object;
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @Override
    public void setDisplayName (final @Nonnull String displayName)
      {
        SkosConceptUtils.setDisplayName(object.as(Entity.class), displayName);
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @Override
    public void setDisplayName (final @Nonnull String displayName,
                                final @Nonnull Locale locale)
      {
        SkosConceptUtils.setDisplayName(object.as(Entity.class), displayName, locale);
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @Override
    public void setDisplayNames (final @Nonnull Map<Locale, String> displayNames)
      {
        for (final Entry<Locale, String> entry : displayNames.entrySet())
          {
            setDisplayName(entry.getValue(), entry.getKey());
          }

        // FIXME: delete eventual other names in locales not specified
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @Override @Nonnull
    public String getDisplayName()
      {
        return SkosConceptUtils.getDisplayName(object.as(Entity.class));
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @Override @Nonnull
    public String getDisplayName (final @Nonnull Locale locale)
      {
        return SkosConceptUtils.getDisplayName(object.as(Entity.class), locale);
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @Override @Nonnull
    public Map<Locale, String> getDisplayNames()
      {
        final Map<Locale, String> result = new HashMap<Locale, String>();

        for (final Locale locale : getLocales())
          {
            result.put(locale, getDisplayName(locale));
          }

        return result;
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @Override @Nonnull
    public SortedSet<Locale> getLocales()
      {
        return new TreeSet<Locale>(Collections.singleton(Locale.ENGLISH));  // FIXME
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @Override
    public void addBroader (final @Nonnull As broader)
      {
        createBroaderNarrowerRelationship(broader, object);
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @Override
    public void addNarrower (final @Nonnull As narrower)
      {
        createBroaderNarrowerRelationship(object, narrower);
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @Override
    @Nonnull
    public <T> Finder<T> findBroaders()
      {
        return new FinderHelper<T>()
          {
            @Override @Nonnull
            protected Set<Concept> doCompute()
              {
                // FIXME: implement as a query
                return object.as(Concept.class).getSkosBroaders();
              }
          };
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @Override
    @Nonnull
    public <T> Finder<T> findNarrowers()
      {
        return new FinderHelper<T>()
          {
            @Override @Nonnull
            protected Set<Concept> doCompute()
              {
                // FIXME: implement as a query
                return object.as(Concept.class).getSkosNarrowers();
              }
          };
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @Override
    public void addPropertyChangeListener(PropertyChangeListener pl)
      {
        // FIXME: TBD
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @Override
    public void removePropertyChangeListener(PropertyChangeListener pl)
      {
        // FIXME: TBD
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    private static void createBroaderNarrowerRelationship (final @Nonnull As broader, final @Nonnull As narrower)
      {
        final Concept unwrappedBroader = Wrapper.unwrap(broader.as(Concept.class));
        final Concept unwrapperNarrower = Wrapper.unwrap(narrower.as(Concept.class));

        unwrappedBroader.getSkosNarrowers().add(unwrapperNarrower);
        unwrapperNarrower.getSkosBroaders().add(unwrappedBroader); // FIXME: see note in Utils.findSameAs()
      }
  }
