/*******************************************************************************
 *
 * blueBill - open source ornithology
 * ==================================
 *
 * Copyright (C) 2008-2009 by Tidalwave s.a.s.
 * Project home page: http://bluebill.tidalwave.it
 *
 *******************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *******************************************************************************
 *
 * $Id$
 *
 ******************************************************************************/
package it.tidalwave.openrdf.elmo;

import it.tidalwave.semantic.SkosConceptUtils;
import java.util.Collections;
import java.util.Locale;
import java.util.SortedSet;
import java.util.TreeSet;
import javax.annotation.Nonnull;
import javax.xml.namespace.QName;
import org.openrdf.elmo.Entity;
import org.openrdf.elmo.LiteralManager;
import org.openrdf.elmo.sesame.SesameManager;
import org.openrdf.model.Literal;
import org.openrdf.model.Resource;
import org.openrdf.model.Statement;
import org.openrdf.model.URI;
import org.openrdf.model.Value;
import org.openrdf.model.impl.URIImpl;
import org.openrdf.repository.RepositoryException;
import org.openrdf.repository.RepositoryResult;

/*******************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id$
 *
 ******************************************************************************/
public class RDFUtils
  {
    private RDFUtils()
      {
      }

    public static void setDisplayName (@Nonnull final Entity entity,
                                       @Nonnull final String displayName)
      {
        final SesameManager em = getSesameManager();
        setDisplayName(entity, displayName, em.getLocale());
      }

    public static void setDisplayName (@Nonnull final Entity entity,
                                       @Nonnull final String displayName,
                                       @Nonnull final Locale locale)
      {
        final SesameManager em = getSesameManager();
        final LiteralManager<URI, Literal> literalManager = em.getLiteralManager();
        final Literal literal = (locale != null) ? literalManager.getLiteral(displayName, locale.getLanguage())
                                                 : literalManager.getLiteral(displayName);
        final URI predicate = new URIImpl("http://www.w3.org/2000/01/rdf-schema#label"); // FIXME
        try
          {
//            final QName qName = entity.getLookup().lookup(Entity.class).getQName();
            final QName qName = entity.getQName();
            final Resource entityResource = em.getResourceManager().createResource(qName);
            em.getConnection().add(entityResource, predicate, literal);
          }
        catch (RepositoryException e)
          {
            throw new RuntimeException(e);
          }
      }

    @Nonnull
    public static String getDisplayName (@Nonnull final Entity entity)
      {
        return getDisplayName(entity, null);
      }

    @Nonnull
    public static String getDisplayName (@Nonnull final Entity entity,
                                         @Nonnull final Locale locale)
      {
        final SesameManager em = getSesameManager();
        final Resource entityResource = em.getResourceManager().createResource(entity.getQName());
//        final Resource entityResource = em.getResourceManager().createResource(((Entity)entity).getQName());
        final URI predicate = new URIImpl("http://www.w3.org/2000/01/rdf-schema#label"); // FIXME

        try
          {
            final RepositoryResult<Statement> statements = em.getConnection().getStatements(entityResource, predicate, null);
            String nonLocalizedValue = "";

            while (statements.hasNext())
              {
                final Statement statement = statements.next();
                final Value object = statement.getObject();

                if (object instanceof Literal)
                  {
                    final Literal literal = (Literal)object;
                    final String literalLanguage = literal.getLanguage();

                    if (literalLanguage == null)
                      {
                        nonLocalizedValue = literal.stringValue();

                        if (locale == null)
                          {
                            return literal.stringValue();
                          }
                      }
                    else if ((locale != null) && literalLanguage.equals(locale.getLanguage()))
                      {
                        return literal.stringValue();
                      }
                  }
              }

            // FIXME: trick since skos:prefLabel is a subclass of rdfs:label.
            // But a generic query with inference would be better in order to support other propertyes such as dc:title etc.
            final String skosLabel = SkosConceptUtils.getDisplayName(entity, locale);

            return skosLabel.equals("") ? nonLocalizedValue : skosLabel;
          }
        catch (RepositoryException e)
          {
            throw new RuntimeException(e);
          }
      }

    @Nonnull
    public static SortedSet<Locale> getLocales (@Nonnull final Entity entity)
      {
        return new TreeSet<Locale>(Collections.singleton(Locale.ENGLISH));  // FIXME
      }
    
    @Nonnull
    private static SesameManager getSesameManager()
      {
        return (SesameManager)ElmoManagerThreadLocal.get();
      }
  }

